clear all; clc; close all;

%Parametros mecnicos
d = 0.18; %posio do CM do brao
M0 = 0.120; %massa do brao
M1 = 0.076; %massa do pndulo
L0 = 0.26; %comprimento do brao
L1 = 0.25; %comprimento do pndulo
I0 = 6.7600e-04; %momento de Inrcia brao
I1 = 3.95833e-04; %momento de Inrcia pndulo
r = 0.25; %distncia da junta do brao  junta do pndulo
b0=0; b1=0; %Coefs de atrito viscoso das juntas de brao e pndulo

%Parmetros do motor
Rm = 3.6; %Resistncia da armadura
Ke = 0.118; %Cte de f.c.e.m. (j considera a reduo)
Kt = 0.118; %Cte de torque (j considera a reduo)

%Variveis de posio, velocidade e acelerao angulares
syms t th0(t) th1(t) g;
th0 = th0(t); th1 = th1(t); %Posio
wth0 = diff(th0,t); wth0_d = diff(wth0,t); %Velocidade
wth1 = diff(th1,t); wth1_d = diff(wth1,t); %Acelerao

% Matrizes de Inrcia
I0_mat = diag([0;0;I0]);
I1_mat = diag([I1;0;0]);

%% Matrizes de rotao
R0 = [cos(th0) -sin(th0) 0; sin(th0) cos(th0) 0; 0 0 1];
R1 = [1 0 0; 0 cos(th1) -sin(th1); 0 sin(th1) cos(th1)];
%% Calculo das velocidades
% brao
w0 = R0.'*[0;0;0] + [0; 0; wth0];
vcm0 = R0.'*[0;0;0] + cross(w0,[d;0;0]);
% pendulo
w1 = R1.'*w0 + [wth1; 0; 0];
v1 = R1.'*cross(w0,[r; 0; 0]);
vcm1 = v1+cross(w1,[0; 0; L1/2]);
%% Clculos das Energias e Lagrangiano
%Energia Cintica do Brao
K0 = 1/2*(w0.' * I0_mat * w0) + 1/2*(vcm0.' * M0 * vcm0);
%Energia Cintica do Pndulo
K1 = 1/2*(w1.' * I1_mat * w1) + 1/2*(vcm1.' * M1 * vcm1);
% Energia cintica total
K = K0+K1;
% Energia potencial total
U = M1*g*L1/2*cos(th1);
% Lagrangiano
Lagr = K - U;
%% Eqs. de Euler-Lagrange
% dLag\d(wth0). deriv(f,g(t)) faz a derivada parcial de f em relao a g(t)
Lqq0 = deriv(Lagr, wth0);
% d(dLag\d(wth0))\dt
Lqqt0 = diff(Lqq0, t);
% dLag\dth0
Lq0 = deriv(Lagr, th0);
% d(dLag\d(wth0))\dt - dLag\dth0
Leq0 = simplify(expand(Lqqt0 - Lq0));

% dLag\d(wth1)
Lqq1 = deriv(Lagr, wth1);
% d(dLag\d(wth1))\dt
Lqqt1 = diff(Lqq1, t);
% dLag\dth1
Lq1 = deriv(Lagr, th1);
% d(dLag\d(wth1))\dt - dLag\dth1
Leq1 = simplify(expand(Lqqt1 - Lq1));

% Como as derivadas j foram feitas, vamos substituir as variveis
% com dependncia temporal por variveis que no dependem do tempo, pois o
% problema agora consiste em resolver equaes algbricas
syms TH0 TH1 WTH0 WTH1 WTH0_d WTH1_d

Leq0_ = subs(Leq0,wth0_d,WTH0_d);
Leq0_ = subs(Leq0_,wth1_d,WTH1_d);
Leq0_ = subs(Leq0_,wth0,WTH0);
Leq0_ = subs(Leq0_,wth1,WTH1);
Leq0_ = subs(Leq0_,th0,TH0);
Leq0_ = subs(Leq0_,th1,TH1);

Leq1_ = subs(Leq1,wth0_d,WTH0_d);
Leq1_ = subs(Leq1_,wth1_d,WTH1_d);
Leq1_ = subs(Leq1_,wth0,WTH0);
Leq1_ = subs(Leq1_,wth1,WTH1);
Leq1_ = subs(Leq1_,th0,TH0);
Leq1_ = subs(Leq1_,th1,TH1);

%% Equaes de Movimento
syms tau_eq_0 tau_eq_1 u %tau_eq_i so as forcas generalizadas e u  um
                          %sinal de -1 a 1 representando o ciclo ativo do
                          %PWM do motor e o sentido de rotao
tau_eq = [tau_eq_0; tau_eq_1];

% Eqs. de Euler Lagrange reagrupadas
s0 = Leq0_ - tau_eq_0;
s1 = Leq1_ - tau_eq_1;
s=[s0; s1];

syms q q_p q_pp
q=[TH0; TH1]; %Vetor de posies angulares (coordenadas generalizadas)
q_p=[WTH0; WTH1]; %Vetor de velocidades angulares
q_pp=[WTH0_d; WTH1_d]; %Vetor de aceleraes angulares

% Modelo Reduzido 
M=simplify(expand(jacobian(s,q_pp))); %Matriz de Inrcia
G=simplify(expand(jacobian(s,g)*g)); %vetor de termos gravitacionais
V=simplify(expand(s-M*q_pp-G+tau_eq)); %vetor de f centripeta-Coriolis
G = subs(G,g,9.81);
tau_m = Kt*(12*u-Ke*WTH0)/(Rm); %Equao do motor DC
tau_eq_0 = tau_m - b0*WTH0; %Torque de atrito da junta do brao
tau_eq_1 = -b1*WTH1; %Torque de atrito da junta do pndulo
tau_eq = subs(tau_eq);

% M, V, G e tau_eq so exportados como funes
% para serem carregadas no modelo no linear do Simulink
matlabFunction(M,'File','function_M_Furuta');
matlabFunction(V,'File','function_V_Furuta');
matlabFunction(G,'File','function_G_Furuta');
matlabFunction(tau_eq,'File','function_tau_eq_Furuta');

% Equaes de Movimento
xpp = simplify(expand(M^-1*(tau_eq-V-G)));
%% Linearizao do Modelo
% variaveis e estado
x = [TH0;TH1;WTH0;WTH1];
xp = [WTH0; WTH1; xpp(1); xpp(2)];

%% Linearizao
Am=jacobian(xp,x); 
Bm=jacobian(xp,u);

% ponto de equilbrio
TH0=0; TH1=0; WTH0=0;WTH1=0;u=0;

A = double(subs(Am));
B = double(subs(Bm));
C = [1 0 0 0]; D = 0;

save('Modelo_Furuta.mat','A','B','C','D');