clear all; close all; clc;
%Parametros mecnicos
M1 = 0.117; %massa da haste
M2 = 0.119; %massa da roda
L = 0.14298; %comptimento da haste
I1 = 6.2533e-4; %momento de inrcia da haste
I2 = 9.4559e-4; %momento de inrcia da roda
d = 0.0987; %distncia eixo base at c.m. da haste
b1 = 0; b2 = 0; %Coefs de atrito viscoso das juntas da haste e da roda

%Parmetros do motor
Rm = 2.44; %Resistncia da armadura
Ke = 0.1836; %Cte de f.c.e.m. (j considera a reduo)
Kt = 0.0601; %Cte de torque (j considera a reduo)

%Variveis de posio, velocidade e acelerao angulares
syms t th1(t) th2(t) g;
th1 = th1(t); th2 = th2(t);
th1_d = diff(th1); th2_d = diff(th2); 
th1_dd = diff(th1_d); th2_dd = diff(th2_d);

% Matrizes de Inrcia
I1_mat = diag([0 0 I1]);
I2_mat = diag([0 0 I2]);

%% Matrizes de rotao
% Haste
R1 = [cos(th1) -sin(th1) 0; sin(th1) cos(th1) 0; 0 0 1];
% Roda
R2 = [cos(th2) -sin(th2) 0; sin(th2) cos(th2) 0; 0 0 1];

%% Calculo das velocidades
% Haste
w1 = simplify(expand(R1.'*[0;0;0] + [0; 0; th1_d]));
vc1 = simplify(expand(cross(w1,[0;d;0])));
% Roda
w2 = simplify(expand(R2.'*w1 + [0; 0; th2_d]));
vc2 = simplify(expand(R2.'*cross(w1,[0;L;0])));

%% Clculos das Energias e Lagrangiano
%Energia Cintica do Rotao (haste + roda)
Ecr = 1/2*w1.'*I1_mat*w1 + 1/2*w2.'*I2_mat*w2;
%Energia Cintica do Translao (haste + roda)
Ect = 1/2*vc1.'*M1*vc1 + 1/2*vc2.'*M2*vc2;
% Energia potencial total (haste + roda)
U = M1*g*d*cos(th1) + M2*g*L*cos(th1);
% Lagrangiano
Lagr = Ecr+Ect-U;

%% Eqs. de Euler-Lagrange
% dL\d(th1_d)              
Lqq1 = deriv(Lagr, th1_d);
% d(dL\d(th1_d))\dt        
Lqqt1 = diff(Lqq1, t);
% dL\dth1               
Lq1 = deriv(Lagr, th1);
% d(dL\d(th1_d))\dt - dL\dth1
Leq1 = simplify(expand(Lqqt1 - Lq1));

% dL\d(th2_d)              
Lqq2 = deriv(Lagr, th2_d);
% d(dL\d(th2_d))\dt        
Lqqt2 = diff(Lqq2, t);
% dL\dth2               
Lq2 = deriv(Lagr, th2);
% d(dL\d(th2_d))\dt - dL\dth2
Leq2 = simplify(expand(Lqqt2 - Lq2));

% Como as derivadas j foram feitas, vamos substituir as variveis
% com dependncia temporal por variveis que no dependem do tempo, pois o
% problema agora consiste em resolver equaes algbricas
syms TH1 TH1_p TH1_pp;
syms TH2 TH2_p TH2_pp;

Leq1_ = subs(Leq1,th1_dd,TH1_pp);
Leq1_ = subs(Leq1_,th1_d,TH1_p);
Leq1_ = subs(Leq1_,th1,TH1);
Leq1_ = subs(Leq1_,th2_dd,TH2_pp);
Leq1_ = subs(Leq1_,th2_d,TH2_p);
Leq1_ = subs(Leq1_,th2,TH2);

Leq2_ = subs(Leq2,th1_dd,TH1_pp);
Leq2_ = subs(Leq2_,th1_d,TH1_p);
Leq2_ = subs(Leq2_,th1,TH1);
Leq2_ = subs(Leq2_,th2_dd,TH2_pp);
Leq2_ = subs(Leq2_,th2_d,TH2_p);
Leq2_ = subs(Leq2_,th2,TH2);

%% Equaes de Movimento
syms tau_eq_1 tau_eq_2 u %tau_eq_i so as forcas generalizadas e u  um
                          %sinal de -1 a 1 representando o ciclo ativo do
                          %PWM do motor e o sentido de rotao
tau_eq = [tau_eq_1; tau_eq_2];

% Eqs. de Euler Lagrange reagrupadas
s1 = Leq1_ - tau_eq_1;
s2 = Leq2_ - tau_eq_2;
s=[s1; s2];

syms q q_p q_pp
q_pp=[TH1_pp; TH2_pp];
q_p=[TH1_p; TH2_p];
q=[TH1; TH2];

% Modelo Reduzido 
M=simplify(expand(jacobian(s,q_pp))); %Matriz de Inrcia
G=simplify(expand(jacobian(s,g)*g)); %vetor de termos gravitacionais
V=simplify(expand(s-M*q_pp-G+tau_eq)); %vetor de f centripeta-Coriolis
G = subs(G,g,9.81);
tau_eq_2 = Kt*(12*u-Ke*TH2_p)/(Rm); %Equao do motor DC
tau_eq_2 = tau_eq_2 - b2*TH2_p; %Torque eq. na junta da roda 
tau_eq_1 = -b1*TH1_p; %Torque eq. na junta do pndulo
tau_eq = subs(tau_eq);

% M, V, G e tau_eq so exportados como funes
% para serem carregadas no modelo no linear do Simulink
matlabFunction(M,'File','function_M_Reacao');
matlabFunction(V,'File','function_V_Reacao');
matlabFunction(G,'File','function_G_Reacao');
matlabFunction(tau_eq,'File','function_tau_eq_Reacao');

% Equaes de Movimento
xpp = simplify(expand(M^-1*(tau_eq-V-G)));
%% Linearizao do Modelo
% variaveis e estado
x = [TH1; TH2; TH1_p; TH2_p];
xp = [TH1_p; TH2_p; xpp(1); xpp(2)];

%% Linearizao
Am=jacobian(xp,x); 
Bm=jacobian(xp,u);

% ponto de equilbrio
TH1=0; TH2=0; TH1_p=0; TH2_p=0; u=0;

A = double(subs(Am));
B = double(subs(Bm));

save('Modelo_Roda_Reacao.mat','A','B');
