clear all; close all; clc;
%Ex. 6.1 - Projeto sem efeito de ZOH(s)
s = tf('s');
G = 1/(s*(s+0.3));
zeta = 0.5; wd = pi; wn = wd/sqrt(1-zeta^2);
s0 = -zeta*wn+sqrt(-1)*wd;
alpha = 0.3; Kc = wn^2; beta = 2*zeta*wn;
C = Kc*(s+alpha)/(s+beta);
Ts = 0.2; z0 = exp(s0*Ts);
Cd1 = zpk(c2d(C,Ts,'matched')); %Controlador discreto na forma ZPK
Kc_1 = Cd1.k; alpha_1 = Cd1.z{1}; beta_1 = Cd1.p{1};

%Ex. 6.1 - Projeto com efeito de ZOH(s)
A = [1 0 -1; 10 0 -2*zeta*wn; 0 -10 wn^2];
d = [2*zeta*wn-10; wn^2; 0];
x2 = A\d; %Ax=d
Kc_ = x2(2); beta_ = x2(1);
C2 = Kc_*(s+alpha)/(s+beta_);
Cd2 = zpk(c2d(C2,Ts,'matched')); %Controlador discreto na forma ZPK
Kc_2 = Cd2.k; alpha_2 = Cd2.z{1}; beta_2 = Cd2.p{1};

%Ex. 6.2 - Discretizar a planta e projetar no plano-z
Gd = zpk(c2d(G,Ts,'zoh'));
K = Gd.K; a = Gd.z{1}; b = Gd.p{1}(2);
temp = conv([1 -z0],[1 -z0']);
gamma = -temp(2); rho = temp(3);
A = [1 -K; 1 -K*a]; d = [gamma-1; rho];
x3 = A\d;
z = tf('z'); z.Ts = Ts;
Kc_3 =x3(2); alpha_3 = b; beta_3 = x3(1); 
Cd3 = Kc_3*(z-alpha_3)/(z-beta_3);

t_run = 6; %Tempo de simulao (ajustar no Simulink)
sim('SIM_Ex_06_01_e_06_02.slx');
t_d = (0:length(u2)-1)*Ts; %Tempo discreto para grfico do sinal de controle
subplot(121);
p1_r = plot(tout,r,'k:'); set(p1_r,'LineWidth',2); hold on;
p1 = plot(tout,y,'k',tout,y1,'b--',tout,y2,'r',tout,y3,'g--');
set(p1,'LineWidth',1.5);
set(gca,'FontName','Times New Roman','Fontsize',12);
title('Resposta ao Degrau Unitrio','FontName',...
    'Times New Roman','Fontsize',14);
xlabel('$t\,\,(s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
ylabel('$y(t)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
leg = legend('$R(s)$','$C_1(s)$','$C_1(z)$','$C_2(z)$','$C_3(z)$');
set(leg,'Fontsize',12,'Interpreter','Latex');
hold on; 

subplot(122);
p2 = plot(tout,u,'k');
set(p2,'LineWidth',1.5);
hold on;
stairs(t_d,u1,'b--','LineWidth',1.5); 
stairs(t_d,u2,'r','LineWidth',1.5); 
stairs(t_d,u3,'g--','LineWidth',1.5);
set(gca,'FontName','Times New Roman','Fontsize',12);
title('Ao de Controle','FontName',...
    'Times New Roman','Fontsize',14);
xlabel('$nT_s \,\,(s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
ylabel('$u(nT_s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);

