clear all; %clc; close all;
s = tf('s'); z = tf('z');
G = 1/((s+1)*(s+2)); %Planta em tempo contnuo
Ts_1 = 0.2; Ts_2 = 0.16; Ts_3 =  0.12; %Valores de Ts
Gd1 = c2d(G,Ts_1,'zoh'); %Discretiza a planta
Gd2 = c2d(G,Ts_2,'zoh'); %Discretiza a planta
Gd3 = c2d(G,Ts_3,'zoh'); %Discretiza a planta
G_MF_1 = z^(-1); 
G_MF_1 = G_MF_1/evalfr(G_MF_1,1); %Primeira GM_F
G_MF_2 = 1*z^(-1) + 2*z^(-2) + 1*z^(-3);
G_MF_2 = minreal(G_MF_2/evalfr(G_MF_2,1)); %Segunda GM_F
G_MF_3 = 1*z^(-1) + 2*z^(-2) + 2*z^(-3) + 1*z^(-4);
G_MF_3 = minreal(G_MF_3/evalfr(G_MF_3,1)); %Terceira GM_F
t_run = 3; %Tempo de simulao

%%  Controlador 1
Cd1 =(1/Gd1)*G_MF_1/(1-G_MF_1);
Cd1 = minreal(Cd1); %Simplifica a F.T. do controlador
Cd1_zpk = zpk(Cd1) 
sos_1 = zp2sos(Cd1_zpk.z{:},Cd1_zpk.p{:},Cd1_zpk.k); % Converte para a
% forma serial com sistemas de ordem 2
b11 = sos_1(1,1:3); a11 = sos_1(1,5:6);
% Simula Controlador 1
sim('SIM_07_03_a.slx');
figure(1); subplot(3,1,1);
n_1 = 0:Ts_1:Ts_1*(length(y_2)-1); %vetor tempo discreto
plot(t, y_1); hold on; stairs(n_1, y_2,'r-*','linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_1)],'Interpreter','Latex');
title('Resposta ao Degrau Unitrio');
ylabel('$y(t)$, $y(nT_s)$','Interpreter','Latex');
xlabel('$t(s)$, $nT_s$','Interpreter','Latex');
L = legend('$y(t)$', '$y(nT_s)$'); set(L,'Interpreter','Latex');
set(findall(gcf,'type','line'),'linewidth',1.5); 
figure(2); subplot(3,1,1);
stairs(n_1,u,'linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_1)],'Interpreter','Latex');
title('Ao de Controle');
ylabel('$u(n T_s)$','Interpreter','Latex');
xlabel('$n T_s$','Interpreter','Latex');

%%  Controlador 2
Cd2 =(1/Gd2)*G_MF_2/(1-G_MF_2);
Cd2 = minreal(Cd2);
Cd2_zpk = zpk(Cd2)
sos_2 = zp2sos(Cd2_zpk.z{:},Cd2_zpk.p{:},Cd2_zpk.k); % ordem 4 (2 sistemas
%de ordem 2 em srie)
b21 = sos_2(1,1:3); a21 = sos_2(1,5:6);
b22 = sos_2(2,1:3); a22 = sos_2(2,5:6);
% Simula Controlador 2
sim('SIM_07_03_b.slx');
figure(1); subplot(3,1,2);
n_2 = 0:Ts_2:Ts_2*(length(y_2)-1); %vetor tempo discreto
plot(t, y_1); hold on; stairs(n_2, y_2,'r-*','linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_2)],'Interpreter','Latex');
title('Resposta ao Degrau Unitrio');
ylabel('$y(t)$, $y(nT_s)$','Interpreter','Latex');
xlabel('$t(s)$, $nT_s$','Interpreter','Latex');
L = legend('$y(t)$', '$y(nT_s)$'); set(L,'Interpreter','Latex');
set(findall(gcf,'type','line'),'linewidth',1.5); 
figure(2); subplot(3,1,2);
stairs(n_2,u,'linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_2)],'Interpreter','Latex');
title('Ao de Controle');
ylabel('$u(n T_s)$','Interpreter','Latex');
xlabel('$n T_s$','Interpreter','Latex');
%%  Controlador 3
Cd3 =(1/Gd3)*G_MF_3/(1-G_MF_3);
Cd3 = minreal(Cd3); 
Cd3_zpk = zpk(Cd3)
sos_3 = zp2sos(Cd3_zpk.z{:},Cd3_zpk.p{:},Cd3_zpk.k); % ordem 5 (3 sistemas
%de ordem 2 em srie, sendo um deles com com coef. de maior ordem nulo)
b31 = sos_3(1,1:3); a31 = sos_3(1,5:6);
b32 = sos_3(2,1:3); a32 = sos_3(2,5:6);
b33 = sos_3(3,1:3); a33 = sos_3(3,5:6);
% Simula Controlador 3
sim('SIM_07_03_c.slx');
figure(1); subplot(3,1,3);
n_3 = 0:Ts_3:Ts_3*(length(y_2)-1); %vetor tempo discreto
plot(t, y_1); hold on; stairs(n_3, y_2,'r-*','linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_3)],'Interpreter','Latex');
title('Resposta ao Degrau Unitrio');
ylabel('$y(t)$, $y(nT_s)$','Interpreter','Latex');
xlabel('$t(s)$, $nT_s$','Interpreter','Latex');
L = legend('$y(t)$', '$y(nT_s)$'); set(L,'Interpreter','Latex');
set(findall(gcf,'type','line'),'linewidth',1.5); 
figure(2); subplot(3,1,3);
stairs(n_3,u,'linewidth',1.5); 
y_lim = get(gca,'YLim');
text(2,0.8*y_lim(2),['$T_s = \,$',num2str(Ts_3)],'Interpreter','Latex');
title('Ao de Controle');
ylabel('$u(n T_s)$','Interpreter','Latex');
xlabel('$n T_s$','Interpreter','Latex');