clear all; close all; clc
s = tf('s'); z = tf('z'); Ts = 1/100;
G = 10/((s+1.5)*(s-0.5)*(s+10)); %Sistema a ser controlado
C = (563.2 * s^3 + 6603 * s^2 + 9901 * s + 1895)/...
    (s^4 + 31.54*s^3 + 446.1*s^2 + 3329*s); %Controlador em tempo contínuo
Cd = c2d(C,Ts,'tustin'); %Discretização do controlador por Tustin
% Forma direta 1 - coefs sem truncamento
b_ = Cd.num{:}; b_0 = b_(1); b = b_(2:end);%[b_0, b]: coefs do numerador
a_ = Cd.den{:}; a = a_(2:end); %[1, a]: coefs do denominador 
% Forma direta 1 - coefs com truncamento
b_T_ = round(b_ *1e4)/1e4; 
b_0_T = b_T_(1); b_T = b_T_(2:end);%[b_0_T, b_T]: coefs do numerador
a_T_ = round(a_ *1e4)/1e4; a_T = a_T_(2:end);%[1, a_T]: coefs do denominador 
% Forma Paralela - coefs com truncamento
%dir2par converte da forma direta para a paralela.
%Fonte: John Proakis (2021). Digital Signal Processing Using MATLAB 
%(https://www.mathworks.com/matlabcentral/fileexchange/
%2189-digital-signal-processing-using-matlab),
%MATLAB Central File Exchange. Retrieved August 10, 2021.
[C,B,A] = dir2par(b_,a_); 
c_TP = round(C*1e4)/1e4;%C_1(z) = c_TP
B_ = round(B*1e4)/1e4; A_ = round(B*1e4)/1e4;
b1_TP_ = B_(1,:); b1_0_TP = b1_TP_(1); b1_TP = b1_TP_(2:end);
b2_TP_ = B_(2,:); b2_0_TP = b2_TP_(1); b2_TP = b2_TP_(2:end);
a1_TP_ = A(1,:); a1_TP = a1_TP_(2:end);
a2_TP_ = A(2,:); a2_TP = a2_TP_(2:end);
%[b1_0_TP, b1_TP]: coefs do numerador de C2(s); 
%[b2_0_TP, b2_TP]: coefs do numerador de C3(s); 
%[1, a1_TP]: coefs do denominador de C2(s); 
%[1, a2_TP]: coefs do denominador de C3(s); 
t_run = 25; sim('SIM_Ex_10_04.slx');
t_d = (0:length(u)-1)*Ts;
subplot(121);
p1_r = plot(t_sim,r,'k:'); set(p1_r,'LineWidth',2); hold on;
p1 = plot(t_sim,y,'b',t_sim,y_T,'r--',t_sim,y_TP,'g:');
set(p1,'LineWidth',1.3);
set(gca,'FontName','Times New Roman','Fontsize',12);
title('Resposta ao Degrau Unitário','FontName',...
    'Times New Roman','Fontsize',14);
xlabel('$t\,\,(s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
ylabel('$y(t)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
hold on; 
axis([0 25 -2 2]);
leg = legend('Refer\^{e}ncia','Forma Direta I s/ truncamento','Forma Direta I c/ truncamento','Forma Paralela c/ truncamento');
set(leg,'Fontsize',12,'Interpreter','Latex');
subplot(122);
p2 = stairs(t_d,u,'b'); hold on; p3 = stairs(t_d,u_T,'r--');
p4 = stairs(t_d,u_TP,'g:');
set(p2,'LineWidth',1.3); set(p3,'LineWidth',1.3); set(p4,'LineWidth',1.5);
axis([0 25 -5 30]);
set(gca,'FontName','Times New Roman','Fontsize',12);
title('Ação de Controle','FontName',...
    'Times New Roman','Fontsize',14);
xlabel('$nT_s \,\,(s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);
ylabel('$u(nT_s)$','Interpreter','Latex','FontName',...
    'Times New Roman','Fontsize',14);